@@ -1,5 +1,7 @@
+# flake8: noqa
 # ActivitySim
 # See full license in LICENSE.txt.
+
 from . import (
     accessibility,
     atwork_subtour_destination,
@@ -8,6 +10,7 @@ from . import (
     atwork_subtour_scheduling,
     auto_ownership,
     cdap,
+    disaggregate_accessibility,
     free_parking,
     initialize,
     initialize_los,
@@ -24,11 +27,15 @@ from . import (
     non_mandatory_scheduling,
     non_mandatory_tour_frequency,
     parking_location_choice,
+    school_escorting,
     stop_frequency,
     summarize,
+    telecommute_frequency,
     tour_mode_choice,
     tour_od_choice,
     tour_scheduling_probabilistic,
+    transit_pass_ownership,
+    transit_pass_subsidy,
     trip_departure_choice,
     trip_destination,
     trip_matrices,
@@ -39,4 +46,5 @@ from . import (
     trip_scheduling_choice,
     vehicle_allocation,
     vehicle_type_choice,
+    work_from_home,
 )
