@@ -1,5 +1,6 @@
 # ActivitySim
 # See full license in LICENSE.txt.
+import io
 import logging
 from builtins import range
 
@@ -96,6 +97,9 @@ def households(households_sample_size, override_hh_ids, trace_hh_id):
         df["sample_rate"] = sample_rate
 
     logger.info("loaded households %s" % (df.shape,))
+    buffer = io.StringIO()
+    df.info(buf=buffer)
+    logger.debug("households.info:\n" + buffer.getvalue())
 
     # replace table function with dataframe
     inject.add_table("households", df)
