@@ -1,8 +1,10 @@
 import glob
 import hashlib
+import logging
 import os
 import shutil
 import sys
+from pathlib import Path
 
 import pkg_resources
 import requests
@@ -53,6 +55,12 @@ def add_create_args(parser):
         help="path to new project directory (default: %(default)s)",
     )
 
+    parser.add_argument(
+        "--link",
+        action="store_true",
+        help="cache and reuse downloaded files via symlinking",
+    )
+
 
 def create(args):
     """
@@ -66,12 +74,15 @@ def create(args):
     if args.list:
 
         list_examples()
-        sys.exit(0)
+        return 0
 
     if args.example:
-
-        get_example(args.example, args.destination)
-        sys.exit(0)
+        try:
+            get_example(args.example, args.destination, link=args.link)
+        except Exception:
+            logging.getLogger().exception("failure in activitysim create")
+            return 101
+        return 0
 
 
 def list_examples():
@@ -86,7 +97,9 @@ def list_examples():
     return ret
 
 
-def get_example(example_name, destination, benchmarking=False):
+def get_example(
+    example_name, destination, benchmarking=False, optimize=True, link=True
+):
     """
     Copy project data to user-specified directory.
 
@@ -95,8 +108,6 @@ def get_example(example_name, destination, benchmarking=False):
     `include` field which is a list of files/folders to include
     in the copied example.
 
-
-
     Parameters
     ----------
 
@@ -107,6 +118,13 @@ def get_example(example_name, destination, benchmarking=False):
         will be copied into a subdirectory with the same name
         as the example
     benchmarking: bool
+    optimize: bool
+    link: bool or path-like
+        Files downloaded via http pointers will be cached in
+        this location.  If a path is not given but just a truthy
+        value, then a cache directory is created using in a location
+        selected by the appdirs library (or, if not installed,
+        linking is skipped.)
     """
     if example_name not in EXAMPLES:
         sys.exit(f"error: could not find example '{example_name}'")
@@ -137,7 +155,7 @@ def get_example(example_name, destination, benchmarking=False):
             sha256 = None
 
         if assets.startswith("http"):
-            download_asset(assets, target_path, sha256)
+            download_asset(assets, target_path, sha256, link=link)
 
         else:
             for asset_path in glob.glob(_example_path(assets)):
@@ -145,6 +163,19 @@ def get_example(example_name, destination, benchmarking=False):
 
     print(f"copied! new project files are in {os.path.abspath(dest_path)}")
 
+    if optimize:
+        optimize_func_names = example.get("optimize", None)
+        if isinstance(optimize_func_names, str):
+            optimize_func_names = [optimize_func_names]
+        if optimize_func_names:
+            from ..examples import optimize_example_data
+
+            for optimize_func_name in optimize_func_names:
+                getattr(
+                    optimize_example_data,
+                    optimize_func_name,
+                )(os.path.abspath(dest_path))
+
     instructions = example.get("instructions")
     if instructions:
         print(instructions)
@@ -153,6 +184,7 @@ def get_example(example_name, destination, benchmarking=False):
 def copy_asset(asset_path, target_path, dirs_exist_ok=False):
 
     print(f"copying {os.path.basename(asset_path)} ...")
+    sys.stdout.flush()
     if os.path.isdir(asset_path):
         target_path = os.path.join(target_path, os.path.basename(asset_path))
         shutil.copytree(asset_path, target_path, dirs_exist_ok=dirs_exist_ok)
@@ -164,44 +196,89 @@ def copy_asset(asset_path, target_path, dirs_exist_ok=False):
         shutil.copy(asset_path, target_path)
 
 
-def download_asset(url, target_path, sha256=None):
+def download_asset(url, target_path, sha256=None, link=True):
+    if link:
+        if not isinstance(link, (str, Path)):
+            try:
+                import appdirs
+            except ImportError:
+                link = False
+            else:
+                link = appdirs.user_data_dir("ActivitySim")
+        original_target_path = target_path
+        target_path = os.path.join(link, target_path)
     os.makedirs(os.path.dirname(target_path), exist_ok=True)
     if url.endswith(".gz") and not target_path.endswith(".gz"):
         target_path_dl = target_path + ".gz"
     else:
         target_path_dl = target_path
+    download = True
     if sha256 and os.path.isfile(target_path):
         computed_sha256 = sha256_checksum(target_path)
         if sha256 == computed_sha256:
             print(f"not re-downloading existing {os.path.basename(target_path)} ...")
-            return
+            download = False
         else:
             print(f"re-downloading existing {os.path.basename(target_path)} ...")
             print(f"   expected checksum {sha256}")
             print(f"   computed checksum {computed_sha256}")
     else:
         print(f"downloading {os.path.basename(target_path)} ...")
-    with requests.get(url, stream=True) as r:
-        r.raise_for_status()
-        with open(target_path_dl, "wb") as f:
-            for chunk in r.iter_content(chunk_size=None):
-                f.write(chunk)
-    if target_path_dl != target_path:
-        import gzip
+    sys.stdout.flush()
+    if download:
+        with requests.get(url, stream=True) as r:
+            r.raise_for_status()
+            print(f"|        as {target_path_dl}")
+            with open(target_path_dl, "wb") as f:
+                for chunk in r.iter_content(chunk_size=None):
+                    f.write(chunk)
+        if target_path_dl != target_path:
+            import gzip
 
-        with gzip.open(target_path_dl, "rb") as f_in:
-            with open(target_path, "wb") as f_out:
-                shutil.copyfileobj(f_in, f_out)
-        os.remove(target_path_dl)
-    computed_sha256 = sha256_checksum(target_path)
-    if sha256 and sha256 != computed_sha256:
-        raise ValueError(
-            f"downloaded {os.path.basename(target_path)} has incorrect checksum\n"
-            f"   expected checksum {sha256}\n"
-            f"   computed checksum {computed_sha256}"
+            with gzip.open(target_path_dl, "rb") as f_in:
+                with open(target_path, "wb") as f_out:
+                    print(f"|  unzip to {target_path}")
+                    shutil.copyfileobj(f_in, f_out)
+            os.remove(target_path_dl)
+        computed_sha256 = sha256_checksum(target_path)
+        if sha256 and sha256 != computed_sha256:
+            raise ValueError(
+                f"downloaded {os.path.basename(target_path)} has incorrect checksum\n"
+                f"   expected checksum {sha256}\n"
+                f"   computed checksum {computed_sha256}"
+            )
+        elif not sha256:
+            print(f"|  computed checksum {computed_sha256}")
+    if link:
+        os.makedirs(
+            os.path.dirname(os.path.normpath(original_target_path)),
+            exist_ok=True,
         )
-    elif not sha256:
-        print(f"   computed checksum {computed_sha256}")
+
+        # check if the original_target_path exists and if so check if it is the correct file
+        if os.path.isfile(os.path.normpath(original_target_path)):
+            if sha256 is None:
+                sha256 = sha256_checksum(os.path.normpath(target_path))
+            existing_sha256 = sha256_checksum(os.path.normpath(original_target_path))
+            if existing_sha256 != sha256:
+                os.unlink(os.path.normpath(original_target_path))
+
+        # if the original_target_path exists now it is the correct file, keep it
+        if not os.path.isfile(os.path.normpath(original_target_path)):
+            try:
+                os.symlink(
+                    os.path.normpath(target_path),
+                    os.path.normpath(original_target_path),
+                )
+            except OSError:
+                # permission errors likely foil symlinking on windows
+                shutil.copy(
+                    os.path.normpath(target_path),
+                    os.path.normpath(original_target_path),
+                )
+                print(f"|    copied to {os.path.normpath(original_target_path)}")
+            else:
+                print(f"| symlinked to {os.path.normpath(original_target_path)}")
 
 
 def sha256_checksum(filename, block_size=65536):
@@ -210,3 +287,18 @@ def sha256_checksum(filename, block_size=65536):
         for block in iter(lambda: f.read(block_size), b""):
             sha256.update(block)
     return sha256.hexdigest()
+
+
+def display_sha256_checksums(directory=None):
+    print("SHA 256 CHECKSUMS")
+    if directory is None:
+        if len(sys.argv) > 1:
+            directory = sys.argv[1]
+        else:
+            directory = os.getcwd()
+    print(f"  in {directory}")
+    for dirpath, dirnames, filenames in os.walk(directory):
+        print(f"- in {dirpath}")
+        for filename in filenames:
+            f = os.path.join(dirpath, filename)
+            print(f"= {sha256_checksum(f)} = {f}")
