@@ -8,10 +8,12 @@ import math
 import multiprocessing
 import os
 import threading
+import warnings
 from contextlib import contextmanager
 
 import numpy as np
 import pandas as pd
+import xarray as xr
 
 from . import config, mem, tracing, util
 from .util import GB
@@ -539,10 +541,10 @@ class ChunkLedger(object):
             elif isinstance(df, pd.DataFrame):
                 elements = util.iprod(df.shape)
                 bytes = 0 if not elements else df.memory_usage(index=True).sum()
-            elif isinstance(df, np.ndarray):
+            elif isinstance(df, (np.ndarray, xr.DataArray)):
                 elements = util.iprod(df.shape)
                 bytes = df.nbytes
-            elif isinstance(df, list):
+            elif isinstance(df, (list, tuple)):
                 # dict of series, dataframe, or ndarray (e.g. assign assign_variables target and temp dicts)
                 elements = 0
                 bytes = 0
@@ -550,7 +552,7 @@ class ChunkLedger(object):
                     e, b = size_it(v)
                     elements += e
                     bytes += b
-            elif isinstance(df, dict):
+            elif isinstance(df, (dict, xr.Dataset)):
                 # dict of series, dataframe, or ndarray (e.g. assign assign_variables target and temp dicts)
                 elements = 0
                 bytes = 0
@@ -581,6 +583,8 @@ class ChunkLedger(object):
             shape = f"list({[x.shape for x in df]})"
         elif isinstance(df, dict):
             shape = f"dict({[v.shape for v in df.values()]})"
+        elif isinstance(df, xr.Dataset):
+            shape = df.dims
         else:
             shape = df.shape
 
@@ -846,7 +850,11 @@ class ChunkSizer(object):
                 )
                 estimated_number_of_chunks = None
 
-                assert chunk_training_mode() != MODE_PRODUCTION
+                if chunk_training_mode() == MODE_PRODUCTION:
+                    warnings.warn(
+                        "ActivitySim is running with a chunk_training_mode of "
+                        f"'production' but initial_row_size is zero in {self.trace_label}"
+                    )
 
         # cum_rows is out of phase with cum_overhead
         # since we won't know observed_chunk_size until AFTER yielding the chunk
@@ -1039,6 +1047,10 @@ def chunk_log(trace_label, chunk_tag=None, base=False):
     # a ChunkSizer class object without actually chunking. This
     # avoids breaking the assertion below.
 
+    if chunk_training_mode() == MODE_CHUNKLESS:
+        yield
+        return
+
     assert base == (len(CHUNK_SIZERS) == 0)
 
     trace_label = f"{trace_label}.chunk_log"
@@ -1073,6 +1085,14 @@ def adaptive_chunked_choosers(choosers, chunk_size, trace_label, chunk_tag=None)
 
     # generator to iterate over choosers
 
+    if chunk_training_mode() == MODE_CHUNKLESS:
+        # The adaptive chunking logic is expensive and sometimes results
+        # in needless data copying.  So we short circuit it entirely
+        # when chunking is disabled.
+        logger.info(f"Running chunkless with {len(choosers)} choosers")
+        yield 0, choosers, trace_label
+        return
+
     chunk_tag = chunk_tag or trace_label
 
     num_choosers = len(choosers.index)
@@ -1154,6 +1174,18 @@ def adaptive_chunked_choosers_and_alts(
         chunk of alternatives for chooser chunk
     """
 
+    if chunk_training_mode() == MODE_CHUNKLESS:
+        # The adaptive chunking logic is expensive and sometimes results
+        # in needless data copying.  So we short circuit it entirely
+        # when chunking is disabled.
+        logger.info(f"Running chunkless with {len(choosers)} choosers")
+        yield 0, choosers, alternatives, trace_label
+        return
+
+    check_assertions = False
+    # set to True if debugging is needed; there are many expensive assertions
+    # to check data quality in here
+
     chunk_tag = chunk_tag or trace_label
 
     num_choosers = len(choosers.index)
@@ -1161,18 +1193,17 @@ def adaptive_chunked_choosers_and_alts(
     assert num_choosers > 0
 
     # alternatives index should match choosers (except with duplicate repeating alt rows)
-    assert choosers.index.equals(
-        alternatives.index[~alternatives.index.duplicated(keep="first")]
-    )
+    if check_assertions:
+        assert choosers.index.equals(
+            alternatives.index[~alternatives.index.duplicated(keep="first")]
+        )
 
     last_repeat = alternatives.index != np.roll(alternatives.index, -1)
 
-    assert (num_choosers == 1) or choosers.index.equals(alternatives.index[last_repeat])
-    assert (
-        "pick_count" in alternatives.columns
-        or choosers.index.name == alternatives.index.name
-    )
-    assert choosers.index.name == alternatives.index.name
+    if check_assertions:
+        assert (num_choosers == 1) or choosers.index.equals(
+            alternatives.index[last_repeat]
+        )
 
     logger.info(
         f"{trace_label} Running adaptive_chunked_choosers_and_alts "
@@ -1210,12 +1241,13 @@ def adaptive_chunked_choosers_and_alts(
             alt_end = alt_chunk_ends[offset + rows_per_chunk]
             alternative_chunk = alternatives[alt_offset:alt_end]
 
-            assert len(chooser_chunk.index) == len(
-                np.unique(alternative_chunk.index.values)
-            )
-            assert (
-                chooser_chunk.index == np.unique(alternative_chunk.index.values)
-            ).all()
+            if check_assertions:
+                assert len(chooser_chunk.index) == len(
+                    np.unique(alternative_chunk.index.values)
+                )
+                assert (
+                    chooser_chunk.index == np.unique(alternative_chunk.index.values)
+                ).all()
 
             logger.info(
                 f"Running chunk {i} of {estimated_number_of_chunks or '?'} "
@@ -1245,6 +1277,14 @@ def adaptive_chunked_choosers_by_chunk_id(
     # all have to be included in the same chunk)
     # FIXME - we pathologically know name of chunk_id col in households table
 
+    if chunk_training_mode() == MODE_CHUNKLESS:
+        # The adaptive chunking logic is expensive and sometimes results
+        # in needless data copying.  So we short circuit it entirely
+        # when chunking is disabled.
+        logger.info(f"Running chunkless with {len(choosers)} choosers")
+        yield 0, choosers, trace_label
+        return
+
     chunk_tag = chunk_tag or trace_label
 
     num_choosers = choosers["chunk_id"].max() + 1
