@@ -2,11 +2,12 @@
 # See full license in LICENSE.txt.
 import logging
 
-import numpy as np
-import pandas as pd
-
 from activitysim.core import assign, config, inject, simulate, tracing
-from activitysim.core.util import assign_in_place
+from activitysim.core.util import (
+    assign_in_place,
+    parse_suffix_args,
+    suffix_expressions_df_str,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -51,6 +52,21 @@ def compute_columns(df, model_settings, locals_dict={}, trace_label=None):
     helper_table_names = model_settings.get("TABLES", [])
     expressions_spec_name = model_settings.get("SPEC", None)
 
+    # Extract suffix for disaggregate accessibilities.
+    # The suffix args can either be passed in the model settings or as part of the config file string.
+    # Awkward, but avoids having to put positional arguments in every single asim function.
+    args = parse_suffix_args(expressions_spec_name)
+
+    expressions_spec_name = args.filename
+    suffix = model_settings.get("SUFFIX", args.SUFFIX)
+    roots = model_settings.get("ROOTS", args.ROOTS)
+
+    assert isinstance(roots, list)
+    assert (suffix is not None and roots) or (suffix is None and not roots), (
+        "Expected to find both 'ROOTS' and 'SUFFIX' in %s, missing one"
+        % model_settings_name
+    )
+
     assert expressions_spec_name is not None, (
         "Expected to find 'SPEC' in %s" % model_settings_name
     )
@@ -62,10 +78,14 @@ def compute_columns(df, model_settings, locals_dict={}, trace_label=None):
     logger.debug(
         f"{trace_label} compute_columns using expression spec file {expressions_spec_name}"
     )
+
     expressions_spec = assign.read_assignment_spec(
         config.config_file_path(expressions_spec_name)
     )
 
+    if suffix is not None and roots:
+        expressions_spec = suffix_expressions_df_str(expressions_spec, suffix, roots)
+
     assert expressions_spec.shape[0] > 0, (
         "Expected to find some assignment expressions in %s" % expressions_spec_name
     )
@@ -84,12 +104,10 @@ def compute_columns(df, model_settings, locals_dict={}, trace_label=None):
     _locals_dict.update(tables)
 
     # FIXME a number of asim model preprocessors want skim_dict - should they request it in model_settings.TABLES?
-    _locals_dict.update(
-        {
-            # 'los': inject.get_injectable('network_los', None),
-            "skim_dict": inject.get_injectable("skim_dict", None),
-        }
-    )
+    if config.setting("sharrow", False):
+        _locals_dict["skim_dict"] = inject.get_injectable("skim_dataset_dict", None)
+    else:
+        _locals_dict["skim_dict"] = inject.get_injectable("skim_dict", None)
 
     results, trace_results, trace_assigned_locals = assign.assign_variables(
         expressions_spec, df, _locals_dict, trace_rows=tracing.trace_targets(df)
