@@ -1,13 +1,11 @@
 # ActivitySim
 # See full license in LICENSE.txt.
 import logging
-from builtins import range
-from math import ceil
 
 import numpy as np
 import pandas as pd
 
-from . import chunk, interaction_simulate, logit, pipeline, tracing
+from . import chunk, config, interaction_simulate, logit, pipeline, tracing
 from .simulate import set_skim_wrapper_targets
 
 logger = logging.getLogger(__name__)
@@ -62,83 +60,32 @@ def make_sample_choices(
             probs = probs[~zero_probs]
             choosers = choosers[~zero_probs]
 
-    cum_probs_array = probs.values.cumsum(axis=1)
-    chunk.log_df(trace_label, "cum_probs_array", cum_probs_array)
-
-    # alt probs in convenient layout to return prob of chose alternative
-    # (same layout as cum_probs_arr)
-    alt_probs_array = probs.values.flatten()
-    chunk.log_df(trace_label, "alt_probs_array", alt_probs_array)
-
     # get sample_size rands for each chooser
     rands = pipeline.get_rn_generator().random_for_df(probs, n=sample_size)
 
     # transform as we iterate over alternatives
     # reshape so rands[i] is in broadcastable (2-D) shape for cum_probs_arr
     # i.e rands[i] is a 2-D array of one alt choice rand for each chooser
-    rands = rands.T.reshape(sample_size, -1, 1)
+    # rands = rands.T #.reshape(sample_size, -1, 1)
     chunk.log_df(trace_label, "rands", rands)
 
-    # the alternative value chosen
-    # WHY SHOULD CHOICES COL HAVE TO BE TYPE INT???
-    # choices_array = np.empty([sample_size, len(choosers)]).astype(int)
-    choices_array = np.empty([sample_size, len(choosers)]).astype(
-        alternatives.index.dtype
+    # TODO: is `sample_choices_maker` more efficient?  The order of samples changes, might change repro-randoms
+    from .choosing import sample_choices_maker_preserve_ordering
+
+    choices_array, choice_probs_array = sample_choices_maker_preserve_ordering(
+        probs.values,
+        rands,
+        alternatives.index.values,
     )
-    # chunk log these later after we populate them...
-
-    # the probability of the chosen alternative
-    choice_probs_array = np.empty([sample_size, len(choosers)])
-    # chunk log these later after we populate them...
-
-    alts = np.tile(alternatives.index.values, len(choosers))
-    chunk.log_df(trace_label, "alts", alts)
-
-    # FIXME - do this all at once rather than iterate?
-    for i in range(sample_size):
-
-        # FIXME - do this in numpy, not pandas?
-
-        # rands for this alt in broadcastable shape
-        r = rands[i]
-
-        # position of first occurrence of positive value
-        positions = np.argmax(cum_probs_array > r, axis=1)
-
-        # FIXME - leave positions as numpy array, not pandas series?
-        # positions is series with the chosen alternative represented as a column index in probs
-        # which is an integer between zero and num alternatives in the alternative sample
-        positions = pd.Series(positions, index=probs.index)
-
-        # need to get from an integer offset into the alternative sample to the alternative index
-        # that is, we want the index value of the row that is offset by <position> rows into the
-        # tranche of this choosers alternatives created by cross join of alternatives and choosers
-
-        # offsets is the offset into model_design df of first row of chooser alternatives
-        offsets = np.arange(len(positions)) * alternative_count
-
-        # choices and choice_probs have one element per chooser and is in same order as choosers
-        choices_array[i] = np.take(alts, positions + offsets)
-        choice_probs_array[i] = np.take(alt_probs_array, positions + offsets)
-
-        del positions
-        del offsets
 
     chunk.log_df(trace_label, "choices_array", choices_array)
     chunk.log_df(trace_label, "choice_probs_array", choice_probs_array)
 
-    del alts
-    chunk.log_df(trace_label, "alts", None)
-    del cum_probs_array
-    chunk.log_df(trace_label, "cum_probs_array", None)
-    del alt_probs_array
-    chunk.log_df(trace_label, "alt_probs_array", None)
-
     # explode to one row per chooser.index, alt_zone_id
     choices_df = pd.DataFrame(
         {
             alt_col_name: choices_array.flatten(order="F"),
-            "rand": rands.flatten(order="F"),
+            "rand": rands.T.flatten(order="F"),
             "prob": choice_probs_array.flatten(order="F"),
             choosers.index.name: np.repeat(np.asanyarray(choosers.index), sample_size),
         }
@@ -170,6 +117,7 @@ def _interaction_sample(
     skims=None,
     locals_d=None,
     trace_label=None,
+    zone_layer=None,
 ):
     """
     Run a MNL simulation in the situation in which alternatives must
@@ -227,6 +175,8 @@ def _interaction_sample(
     """
 
     have_trace_targets = tracing.has_trace_targets(choosers)
+    trace_ids = None
+    trace_rows = None
     num_choosers = len(choosers.index)
 
     assert num_choosers > 0
@@ -251,76 +201,159 @@ def _interaction_sample(
 
     chooser_index_id = interaction_simulate.ALT_CHOOSER_ID if log_alt_losers else None
 
+    sharrow_enabled = config.setting("sharrow", False)
+
     # - cross join choosers and alternatives (cartesian product)
     # for every chooser, there will be a row for each alternative
     # index values (non-unique) are from alternatives df
     alternative_count = alternatives.shape[0]
-    interaction_df = logit.interaction_dataset(
-        choosers,
-        alternatives,
-        sample_size=alternative_count,
-        chooser_index_id=chooser_index_id,
-    )
 
-    chunk.log_df(trace_label, "interaction_df", interaction_df)
+    interaction_utilities = None
+    interaction_utilities_sh = None
+    if sharrow_enabled:
 
-    assert alternative_count == len(interaction_df.index) / len(choosers.index)
-
-    if skims is not None:
-        set_skim_wrapper_targets(interaction_df, skims)
-
-    # evaluate expressions from the spec multiply by coefficients and sum
-    # spec is df with one row per spec expression and one col with utility coefficient
-    # column names of interaction_df match spec index values
-    # utilities has utility value for element in the cross product of choosers and alternatives
-    # interaction_utilities is a df with one utility column and one row per row in interaction_df
-    if have_trace_targets:
-        trace_rows, trace_ids = tracing.interaction_trace_rows(
-            interaction_df, choosers, alternative_count
+        (
+            interaction_utilities,
+            trace_eval_results,
+        ) = interaction_simulate.eval_interaction_utilities(
+            spec,
+            choosers,
+            locals_d,
+            trace_label,
+            trace_rows,
+            estimator=None,
+            log_alt_losers=log_alt_losers,
+            extra_data=alternatives,
+            zone_layer=zone_layer,
+        )
+        chunk.log_df(trace_label, "interaction_utilities", interaction_utilities)
+        if sharrow_enabled == "test" or True:
+            interaction_utilities_sh, trace_eval_results_sh = (
+                interaction_utilities,
+                trace_eval_results,
+            )
+    if not sharrow_enabled or (sharrow_enabled == "test"):
+        interaction_df = logit.interaction_dataset(
+            choosers,
+            alternatives,
+            sample_size=alternative_count,
+            chooser_index_id=chooser_index_id,
         )
 
-        tracing.trace_df(
-            interaction_df[trace_rows],
-            tracing.extend_trace_label(trace_label, "interaction_df"),
-            slicer="NONE",
-            transpose=False,
+        chunk.log_df(trace_label, "interaction_df", interaction_df)
+
+        assert alternative_count == len(interaction_df.index) / len(choosers.index)
+
+        if skims is not None:
+            set_skim_wrapper_targets(interaction_df, skims)
+
+        # evaluate expressions from the spec multiply by coefficients and sum
+        # spec is df with one row per spec expression and one col with utility coefficient
+        # column names of interaction_df match spec index values
+        # utilities has utility value for element in the cross product of choosers and alternatives
+        # interaction_utilities is a df with one utility column and one row per row in interaction_df
+        if have_trace_targets:
+            trace_rows, trace_ids = tracing.interaction_trace_rows(
+                interaction_df, choosers, alternative_count
+            )
+
+            tracing.trace_df(
+                interaction_df[trace_rows],
+                tracing.extend_trace_label(trace_label, "interaction_df"),
+                slicer="NONE",
+                transpose=False,
+            )
+        else:
+            trace_rows = trace_ids = None
+
+        # interaction_utilities is a df with one utility column and one row per interaction_df row
+        (
+            interaction_utilities,
+            trace_eval_results,
+        ) = interaction_simulate.eval_interaction_utilities(
+            spec,
+            interaction_df,
+            locals_d,
+            trace_label,
+            trace_rows,
+            estimator=None,
+            log_alt_losers=log_alt_losers,
+            zone_layer=zone_layer,
         )
-    else:
-        trace_rows = trace_ids = None
+        chunk.log_df(trace_label, "interaction_utilities", interaction_utilities)
 
-    # interaction_utilities is a df with one utility column and one row per interaction_df row
-    (
-        interaction_utilities,
-        trace_eval_results,
-    ) = interaction_simulate.eval_interaction_utilities(
-        spec,
-        interaction_df,
-        locals_d,
-        trace_label,
-        trace_rows,
-        estimator=None,
-        log_alt_losers=log_alt_losers,
-    )
-    chunk.log_df(trace_label, "interaction_utilities", interaction_utilities)
+        # ########### HWM - high water mark (point of max observed memory usage)
 
-    # ########### HWM - high water mark (point of max observed memory usage)
+        del interaction_df
+        chunk.log_df(trace_label, "interaction_df", None)
 
-    del interaction_df
-    chunk.log_df(trace_label, "interaction_df", None)
+    if sharrow_enabled == "test":
+        try:
+            if interaction_utilities_sh is not None:
+                np.testing.assert_allclose(
+                    interaction_utilities_sh.values.reshape(
+                        interaction_utilities.values.shape
+                    ),
+                    interaction_utilities.values,
+                    rtol=1e-2,
+                    atol=0,
+                    err_msg="utility not aligned",
+                    verbose=True,
+                )
+        except AssertionError as err:
+            print(err)
+            misses = np.where(
+                ~np.isclose(
+                    interaction_utilities_sh.values,
+                    interaction_utilities.values,
+                    rtol=1e-2,
+                    atol=0,
+                )
+            )
+            _sh_util_miss1 = interaction_utilities_sh.values[
+                tuple(m[0] for m in misses)
+            ]
+            _u_miss1 = interaction_utilities.values[tuple(m[0] for m in misses)]
+            diff = _sh_util_miss1 - _u_miss1
+            if len(misses[0]) > interaction_utilities_sh.values.size * 0.01:
+                print("big problem")
+                print(misses)
+                if "nan location mismatch" in str(err):
+                    print("nan location mismatch interaction_utilities_sh")
+                    print(np.where(np.isnan(interaction_utilities_sh.values)))
+                    print("nan location mismatch interaction_utilities legacy")
+                    print(np.where(np.isnan(interaction_utilities.values)))
+                print("misses =>", misses)
+                j = 0
+                while j < len(misses[0]):
+                    print(
+                        f"miss {j} {tuple(m[j] for m in misses)}:",
+                        interaction_utilities_sh.values[tuple(m[j] for m in misses)],
+                        "!=",
+                        interaction_utilities.values[tuple(m[j] for m in misses)],
+                    )
+                    j += 1
+                    if j > 10:
+                        break
+                raise
 
-    if have_trace_targets:
+    if have_trace_targets and trace_ids is not None:
         tracing.trace_interaction_eval_results(
             trace_eval_results,
             trace_ids,
             tracing.extend_trace_label(trace_label, "eval"),
         )
 
-        tracing.trace_df(
-            interaction_utilities[trace_rows],
-            tracing.extend_trace_label(trace_label, "interaction_utilities"),
-            slicer="NONE",
-            transpose=False,
-        )
+    if have_trace_targets and trace_rows is not None:
+        try:
+            tracing.trace_df(
+                interaction_utilities[trace_rows],
+                tracing.extend_trace_label(trace_label, "interaction_utilities"),
+                slicer="NONE",
+                transpose=False,
+            )
+        except ValueError:
+            pass
 
     tracing.dump_df(DUMP, interaction_utilities, trace_label, "interaction_utilities")
 
@@ -457,6 +490,7 @@ def interaction_sample(
     chunk_size=0,
     chunk_tag=None,
     trace_label=None,
+    zone_layer=None,
 ):
 
     """
@@ -542,6 +576,7 @@ def interaction_sample(
             skims=skims,
             locals_d=locals_d,
             trace_label=chunk_trace_label,
+            zone_layer=zone_layer,
         )
 
         if choices.shape[0] > 0:
